@@ -99,8 +99,8 @@ class TailsInstallerCreator(object):
             if os.getuid() != 0:
                 print >> sys.stderr, _("You must run this application as root")
                 sys.exit(1)
-        self._setup_logger()
         self._error_log_filename = self._setup_error_log_file()
+        self._setup_logger()
 
     def _setup_error_log_file(self):
         temp = tempfile.NamedTemporaryFile(mode='a', delete=False,
@@ -111,10 +111,16 @@ class TailsInstallerCreator(object):
     def _setup_logger(self):
         self.log = logging.getLogger()
         self.log.setLevel(logging.DEBUG if self.opts.verbose else logging.INFO)
-        self.handler = logging.StreamHandler()
-        formatter = logging.Formatter("[%(filename)s:%(lineno)s (%(funcName)s)] %(levelname)s: %(message)s")
-        self.handler.setFormatter(formatter)
-        self.log.addHandler(self.handler)
+
+        formatter = logging.Formatter("%(asctime)s [%(filename)s:%(lineno)s (%(funcName)s)] %(levelname)s: %(message)s")
+
+        self.stream_handler = logging.StreamHandler()
+        self.stream_handler.setFormatter(formatter)
+        self.log.addHandler(self.stream_handler)
+
+        self.file_handler = logging.FileHandler(self._error_log_filename)
+        self.file_handler.setFormatter(formatter)
+        self.log.addHandler(self.file_handler)
 
     def detect_supported_drives(self, callback=None):
         """ This method should populate self.drives with supported devices """
@@ -288,8 +294,10 @@ class TailsInstallerCreator(object):
         err = unicode_to_utf8(err)
         self.output.write(out + '\n' + err + '\n')
         if proc.returncode:
-            self.write_log()
-            if not passive:
+            if passive:
+                self.log.debug(self.output.getvalue())
+            else:
+                self.log.info(self.output.getvalue())
                 raise TailsInstallerError(_(
                         "There was a problem executing the following command: `%(command)s`.\nA more detailed error log has been written to '%(filename)s'.")
                         % {'command': cmd, 'filename': self._error_log_filename})
@@ -458,11 +466,6 @@ class TailsInstallerCreator(object):
                                          " previous LiveOS: %(message)s")
                                          % {'message': str(e)})
 
-    def write_log(self):
-        """ Write out our subprocess stdout/stderr to a log file """
-        with open(self._error_log_filename, 'a') as out:
-            out.write(self.output.getvalue())
-
     def get_liveos(self):
         return self.get_liveos_file_path(config['main_liveos_dir'])
 
@@ -551,7 +554,7 @@ class TailsInstallerCreator(object):
         /org/freedesktop/UDisks2/devices/sdb) from which the system
         is running."""
         liveos_mountpoint = self.running_liveos_mountpoint()
-        if os.path.exists(liveos_mountpoint):
+        if is_running_from_tails() and os.path.exists(liveos_mountpoint):
             return underlying_physical_device(liveos_mountpoint)
         else:
             return False
@@ -820,10 +823,8 @@ class LinuxTailsInstallerCreator(TailsInstallerCreator):
         self.log.debug("Creating partition table")
         # Use udisks instead of plain sgdisk will allow unprivileged users
         # to get a refreshed partition table from the kernel
-        obj = self._get_object()
-        block = obj.props.block
         try:
-            block.call_format_sync(
+            self._get_object().props.block.call_format_sync(
                     'gpt',
                     arg_options=GLib.Variant('a{sv}', None),
                     cancellable=None)
@@ -835,7 +836,7 @@ class LinuxTailsInstallerCreator(TailsInstallerCreator):
                 self.log.debug("Failed to synchronize. Trying again, which usually solves the issue. Error was: %s" % e.message)
                 self.flush_buffers(silent=True)
                 time.sleep(5)
-                block.call_format_sync(
+                self._get_object().props.block.call_format_sync(
                         'gpt',
                         arg_options=GLib.Variant('a{sv}', None),
                         cancellable=None)
@@ -843,7 +844,7 @@ class LinuxTailsInstallerCreator(TailsInstallerCreator):
                 raise
 
         self.log.debug("Creating partition")
-        partition_table = obj.props.partition_table
+        partition_table = self._get_object().props.partition_table
         try:
             partition_table.call_create_partition_sync(
                     arg_offset=0,
@@ -862,37 +863,36 @@ class LinuxTailsInstallerCreator(TailsInstallerCreator):
             else:
                 raise
 
-        self.flush_buffers(silent=True)
-        time.sleep(5)
-
         # Rescan the device as it seems this is not always done automatically.
-        # XXX: this should only be needed when call_create_partition_sync fails.
-        block.call_rescan_sync(GLib.Variant('a{sv}', None))
-
-        # Gets the new object after the rescan, then its 1st partition
+        # And get the new object after the rescan, then its 1st partition
         # which is the one we just created on our new partition table.
-        obj = self._get_object(self.drive['udi'])
-        partition_table = obj.props.partition_table
-        partitions = self._udisksclient.get_partitions(partition_table)
-        self.log.debug("partitions: %(partitions)s" % {'partitions': partitions})
-        system_partition = partitions[0]
+        self.rescan_block_device(self._get_object().props.block)
+        system_partition = self.first_partition(self.drive['udi'])
 
         # Sets the partition type that is not set by call_create_partition_sync...
         # because it is not implemented in UDisks as of version 2.1.5 (see
         # http://cgit.freedesktop.org/udisks/tree/src/udiskslinuxpartitiontable.c?id=a66691c3#n630)
         #
-        # XXX: this sometimes fails with:
-        # tails_installer.creator.py, line 940, in partition_device
-        #    system_partition.call_set_type_sync(ESP_GUID,
-        #        GLib.Variant('a{sv}', None))
-        # Error: udisks-error-quark:
-        # GDBus.Error:org.freedesktop.UDisks2.Error.NotSupported:
-        # No support for modifying a partition a table of type `PMBR' (11)
+        # XXX: sometimes fails (https://labs.riseup.net/code/issues/10987)
         system_partition.call_set_type_sync(ESP_GUID, GLib.Variant('a{sv}', None))
-        system_partition.call_set_name_sync(self.label, GLib.Variant('a{sv}', None))
 
+        # Get a fresh system_partition object, otherwise
+        # _set_partition_flags sometimes fails with
+        # "GDBus.Error:org.freedesktop.DBus.Error.UnknownMethod: No
+        # such interface 'org.freedesktop.UDisks2.Partition' on object
+        # at path /org/freedesktop/UDisks2/block_devices/sda1"
+        self.rescan_block_device(self._get_object().props.block)
+        system_partition = self.first_partition(self.drive['udi'])
+
+        # XXX: sometimes fails (https://labs.riseup.net/code/issues/10987)
         self._set_partition_flags(system_partition, SYSTEM_PARTITION_FLAGS)
 
+        # Give the system some more time to recognize the updated
+        # partition, otherwise sometimes later on, when
+        # switch_drive_to_system_partition is called, it calls
+        # _set_drive, that fails with "Cannot find device /dev/sda1".
+        self.rescan_block_device(self._get_object().props.block)
+
     def update_system_partition_properties(self):
         self.log.info(_('Updating properties of system partition %(system_partition)s') % {
             'system_partition': self.drive['device']
@@ -1002,6 +1002,7 @@ class LinuxTailsInstallerCreator(TailsInstallerCreator):
             for com32mod in com32modules:
                 copied = False
                 for orig_dir in ('/usr/lib/syslinux/modules/bios',
+                                 '/usr/lib/syslinux/bios',
                                  '/usr/share/syslinux',
                                  '/usr/lib/syslinux'):
                     com32path = os.path.join(orig_dir, com32mod)
@@ -1073,6 +1074,15 @@ class LinuxTailsInstallerCreator(TailsInstallerCreator):
             udi = self.drive['udi']
         return self._udisksclient.get_object(udi)
 
+    def first_partition(self, udi=None):
+        """Return the UDisks2.Partition object for the first partition on the drive"""
+        if not udi:
+            udi = self.drive['udi']
+        obj = self._get_object(udi)
+        partition_table = obj.props.partition_table
+        partitions = self._udisksclient.get_partitions(partition_table)
+        return partitions[0]
+
     def terminate(self):
         for pid in self.pids:
             try:
@@ -1232,6 +1242,7 @@ class LinuxTailsInstallerCreator(TailsInstallerCreator):
             for mbr_bin in ('/usr/lib/syslinux/mbr/gptmbr.bin',
                             '/usr/lib/SYSLINUX/gptmbr.bin',
                             '/usr/lib/syslinux/gptmbr.bin',
+                            '/usr/lib/syslinux/bios/gptmbr.bin',
                             '/usr/share/syslinux/gptmbr.bin'):
                 if os.path.exists(mbr_bin):
                     return mbr_bin
@@ -1303,6 +1314,12 @@ class LinuxTailsInstallerCreator(TailsInstallerCreator):
             self.log.info(_("Synchronizing data on disk..."))
         self.popen('sync')
 
+    def rescan_block_device(self, block):
+        self._udisksclient.settle()
+        self.flush_buffers(silent=True)
+        time.sleep(5)
+        block.call_rescan_sync(GLib.Variant('a{sv}', None))
+
     def is_admin(self):
         return os.getuid() == 0
 
